/********************************************************************************** 
  This file is part of the kde-game 'ktron'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <qkeycode.h>
#include <kmsgbox.h>
#include <klocale.h>
#include <kiconloader.h>

#include "app.h"


App::App(const char *name) : KTopLevelWidget(name)
{
  setUnsavedData(TRUE); // because of not saving the game

  setMinimumSize(150,200);
  initmenu();
  tron=new Tron(this);
  connect(tron,SIGNAL(gameEnds(int)),SLOT(changeStatus(int)));
  connect(tron,SIGNAL(sizeChanged(bool)),SLOT(placeTronWidget(bool)));
  connect(tron,SIGNAL(gameReset()),SLOT(removeWinnerMessage()));

  if(!kapp->isRestored())
    {
      KConfig *config=kapp->getConfig();
      if(config != 0)
	{
	  KConfigGroupSaver saver(config,QString());
	  int menuPos = config->readNumEntry("MenubarPos",(int)(KMenuBar::Top));
	  menu->setMenuBarPos((KMenuBar::menuPosition)menuPos);
	  
	  int barPos = config->readNumEntry("ToolbarPos",(int)(KToolBar::Top));
	  toolbar->setBarPos((KToolBar::BarPosition)barPos);
	  
	  bool visible=(bool)(config->readNumEntry("Toolbar",(int)(TRUE)));
	  viewMenu->setItemChecked(ID_VIEW_TOOLBAR,visible);
	  if(visible)
	    enableToolBar(KToolBar::Show);
	  else 
	    enableToolBar(KToolBar::Hide);
	  
	  visible=(bool)(config->readNumEntry("Statusbar",(int)(TRUE)));
	  viewMenu->setItemChecked(ID_VIEW_STATUSBAR,visible);
	  if(visible)
	    enableStatusBar(KStatusBar::Show);
	  else 
	    enableStatusBar(KStatusBar::Hide);
	  
	  bool status=(bool)(config->readNumEntry("Computerplayer1",(int)(FALSE)));
	  compPlayers->setItemChecked(ID_COMP_BASE,status);
	  tron->setComputerplayer(0,status);
	  status=(bool)(config->readNumEntry("Computerplayer2",(int)(FALSE)));
	  compPlayers->setItemChecked(ID_COMP_BASE+1,status);
	  tron->setComputerplayer(1,status);

	  int velocity=config->readNumEntry("Velocity",5);
	  tron->setVelocity(velocity);
	  updateVelocityMenu(ID_VELOCITY_BASE+velocity);

	  TronStyle newStyle=(TronStyle)config->readNumEntry("Style",(int) ORect);
	  tron->setStyle(newStyle);
	  for(int i=ID_STYLE_BASE;i<ID_STYLE_BASE+3;i++)
	    styleMenu->setItemChecked(i,FALSE);
	  
	  switch(newStyle)
	    {
	    case ORect:
	      styleMenu->setItemChecked(ID_STYLE_BASE,TRUE);
	      break;
	    case Rect:
	      styleMenu->setItemChecked(ID_STYLE_BASE+1,TRUE);
	      break;
	    case Circle:
	      styleMenu->setItemChecked(ID_STYLE_BASE+2,TRUE);
	      break;
	    case Pix:
	      styleMenu->setItemChecked(ID_STYLE_BASE+3,TRUE);
	      break;
	    }
	  
	  int size=config->readNumEntry("RectSize",10);
	  tron->setRectSize(size);
	  sizeMenu->setItemChecked(ID_SIZE_BASE+size,TRUE);
	  

	  int width=config->readNumEntry("Width",400);
	  int height=config->readNumEntry("Height",300);
	  resize(width,height);

	}
      else
	{
	  tron->setStyle(ORect);
	  styleMenu->setItemChecked(ID_STYLE_BASE,TRUE);
	  tron->setVelocity(5);
	  updateVelocityMenu(ID_VELOCITY_BASE+5);
	  tron->setRectSize(10);
	  sizeMenu->setItemChecked(ID_SIZE_BASE+10,TRUE);
	}

      tron->show();
      setView(tron);
      show();
    }
  else
    {
      tron->show();
      setView(tron);
    }
  
}

// Destruktor

App::~App()
{ 
  delete viewMenu;
  delete styleMenu;
  delete sizeMenu;
  delete options;
  delete menu;
  delete statusbar;
  delete compPlayers;
  if(tron) delete tron;
}


void App::initmenu()
{
  // Menleiste erstellen

  QPopupMenu *game = new QPopupMenu;
  game->insertItem(klocale->translate("&Pause/Continue"),ID_GAME_PAUSE);
  game->insertItem(klocale->translate("&New Game"),ID_GAME_NEW);
  game->insertSeparator();
  game->insertItem(klocale->translate("&Quit"),ID_GAME_QUIT);

  game->setAccel(CTRL+Key_P,ID_GAME_PAUSE);
  game->setAccel(CTRL+Key_N,ID_GAME_NEW);
  game->setAccel(CTRL+Key_Q,ID_GAME_QUIT);

  connect(game,SIGNAL(activated(int)),SLOT(menuCallback(int)));

   
  viewMenu=new QPopupMenu;
  viewMenu->setCheckable(TRUE);
  viewMenu->insertItem(klocale->translate("Show &Toolbar"),ID_VIEW_TOOLBAR);
  viewMenu->insertItem(klocale->translate("Show &Statusbar"),ID_VIEW_STATUSBAR);
 
  connect(viewMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));


  /* options-menu  */

  compPlayers =new QPopupMenu;
  compPlayers->setCheckable(TRUE);
  compPlayers->insertItem(klocale->translate("Player &1"),ID_COMP_BASE);
  compPlayers->insertItem(klocale->translate("Player &2"),ID_COMP_BASE+1);
  
  connect(compPlayers,SIGNAL(activated(int)),SLOT(menuCallback(int)));


  velocity=new QPopupMenu;
  velocity->setCheckable(TRUE);
  velocity->insertItem(klocale->translate("&1 (slow)"),ID_VELOCITY_BASE+1);
  velocity->insertItem("&2",ID_VELOCITY_BASE+2);
  velocity->insertItem("&3",ID_VELOCITY_BASE+3);
  velocity->insertItem("&4",ID_VELOCITY_BASE+4);
  velocity->insertItem(klocale->translate("&5 (default)"),ID_VELOCITY_BASE+5);
  velocity->insertItem("&6",ID_VELOCITY_BASE+6);
  velocity->insertItem("&7",ID_VELOCITY_BASE+7);
  velocity->insertItem("&8",ID_VELOCITY_BASE+8);
  velocity->insertItem(klocale->translate("&9 (fast)"),ID_VELOCITY_BASE+9);

  velocity->setAccel(CTRL+Key_1,ID_VELOCITY_BASE+1);
  velocity->setAccel(CTRL+Key_2,ID_VELOCITY_BASE+2);
  velocity->setAccel(CTRL+Key_3,ID_VELOCITY_BASE+3);
  velocity->setAccel(CTRL+Key_4,ID_VELOCITY_BASE+4);
  velocity->setAccel(CTRL+Key_5,ID_VELOCITY_BASE+5);
  velocity->setAccel(CTRL+Key_6,ID_VELOCITY_BASE+6);
  velocity->setAccel(CTRL+Key_7,ID_VELOCITY_BASE+7);
  velocity->setAccel(CTRL+Key_8,ID_VELOCITY_BASE+8);
  velocity->setAccel(CTRL+Key_9,ID_VELOCITY_BASE+9);

  connect(velocity,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  QPopupMenu *colors=new QPopupMenu;
  colors->insertItem(klocale->translate("Player &1"),ID_COLOR_BASE+1); 
  colors->insertItem(klocale->translate("Player &2"),ID_COLOR_BASE+2); 
  colors->insertItem(klocale->translate("&Frame"),ID_COLOR_BASE+3);
  colors->insertItem(klocale->translate("&Background"),ID_COLOR_BASE);
 
  colors->setAccel(ALT+Key_1,ID_COLOR_BASE+1);
  colors->setAccel(ALT+Key_2,ID_COLOR_BASE+2);
  colors->setAccel(CTRL+Key_B,ID_COLOR_BASE);
  colors->setAccel(CTRL+Key_F,ID_COLOR_BASE+3);

  connect(colors,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  
  styleMenu=new QPopupMenu;
  styleMenu->setCheckable(TRUE);
  styleMenu->insertItem(klocale->translate("3&D Rect"),ID_STYLE_BASE);
  styleMenu->insertItem(klocale->translate("&Rect"),ID_STYLE_BASE+1);
  styleMenu->insertItem(klocale->translate("&Circle"),ID_STYLE_BASE+2);
  styleMenu->insertItem(klocale->translate("&Pixmap"),ID_STYLE_BASE+3);

  connect(styleMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));


  sizeMenu=new QPopupMenu;
  sizeMenu->setCheckable(TRUE);
  sizeMenu->insertItem(klocale->translate("&very small"),ID_SIZE_BASE+4);
  sizeMenu->insertItem(klocale->translate("&small"),ID_SIZE_BASE+7);
  sizeMenu->insertItem(klocale->translate("&medium"),ID_SIZE_BASE+10);
  sizeMenu->insertItem(klocale->translate("&large"),ID_SIZE_BASE+13);
  sizeMenu->insertItem(klocale->translate("v&ery large"),ID_SIZE_BASE+16);

  connect(sizeMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));
  

  options = new QPopupMenu;
  options->insertItem(klocale->translate("&Computerplayer"),compPlayers);
  options->insertItem(klocale->translate("&Velocity"),velocity);
  options->insertItem(klocale->translate("&Colors"),colors);
  options->insertItem(klocale->translate("&Style"),styleMenu);
  options->insertItem(klocale->translate("Si&ze"),sizeMenu);
  options->insertItem(klocale->translate("&Keys"),ID_OPT_KEYS);
  options->insertSeparator();
  options->insertItem(klocale->translate("&Save Settings"),ID_OPT_SAVE);

  options->setAccel(CTRL+Key_K,ID_OPT_KEYS);

  connect(options,SIGNAL(activated(int)),SLOT(menuCallback(int)));
   

  QPopupMenu *help = new QPopupMenu;
  help->insertItem( klocale->translate("&Contents"),ID_HELP_CONTENTS);
  help->insertItem( klocale->translate("&About..."),ID_HELP_ABOUT);

  help->setAccel(Key_F1,ID_HELP_CONTENTS);

  connect(help,SIGNAL(activated(int)),SLOT(menuCallback(int)));
  
  menu = new KMenuBar( this );
  CHECK_PTR( menu );
  menu->insertItem( klocale->translate("&Game"), game );
  menu->insertItem(klocale->translate("&View"),viewMenu);
  menu->insertItem(klocale->translate("&Options"),options);
  menu->insertSeparator();
  menu->insertItem( klocale->translate("&Help"), help );

  menu->enableFloating(FALSE);

  connect(menu,SIGNAL(moved(menuPosition)),SLOT(barPositionChanged()));
 
  menu->show();
  setMenu(menu);

  
  // Toolbar
 
  KIconLoader *loader = kapp->getIconLoader();

  toolbar = new KToolBar(this);
  CHECK_PTR( toolbar);
  //exitbutton
  toolbar->insertButton(loader->loadIcon("exit.xpm"),ID_GAME_QUIT,
			TRUE,klocale->translate("Quit"));
  // restartbutton
  toolbar->insertButton(loader->loadIcon("reload.xpm"),ID_GAME_NEW,
			TRUE,klocale->translate("New Game"));
  toolbar->insertSeparator();
  //helpbutton
  toolbar->insertButton(loader->loadIcon("help.xpm"),ID_HELP_CONTENTS,TRUE,
			klocale->translate("Help"));
    
  connect(toolbar, SIGNAL(clicked(int)), this, SLOT(menuCallback(int)));
  connect(toolbar,SIGNAL(moved(BarPosition)),SLOT(barPositionChanged()));
 
  toolbar->show();
  addToolBar(toolbar);


  // Statusleiste erstellen

  statusbar=new KStatusBar(this);
  CHECK_PTR( statusbar );
  statusbar->insertItem(klocale->translate("Player 1: 0  "),ID_STATUS_BASE+1);
  statusbar->insertItem(klocale->translate("Player 2: 0  "),ID_STATUS_BASE+2);
  statusbar->insertItem(klocale->translate("                               ")
					   ,ID_STATUS_BASE+3);
  statusbar->show();
  setStatusBar(statusbar);

}

void App::updateVelocityMenu(int id)
{
  for(int i=1;i<10;i++)
    velocity->setItemChecked(ID_VELOCITY_BASE+i,FALSE);
   
  velocity->setItemChecked(id,TRUE);
}


void App::saveSettings()
{
  KConfig *config=kapp->getConfig();
  KConfigGroupSaver saver(config,QString());

  bool status=viewMenu->isItemChecked(ID_VIEW_TOOLBAR);
  config->writeEntry("Toolbar",(int)( status ));
  status=viewMenu->isItemChecked(ID_VIEW_STATUSBAR);
  config->writeEntry("Statusbar",(int)( status ));
  
  config->writeEntry("Width",width());
  config->writeEntry("Height",height());
  config->writeEntry("Velocity",tron->getVelocity());
  config->writeEntry("Style",(int)tron->getStyle());
  config->writeEntry("RectSize",tron->getRectSize());

  config->writeEntry("Computerplayer1"
		     ,(int)compPlayers->isItemChecked(ID_COMP_BASE));
  config->writeEntry("Computerplayer2"
		     ,(int)compPlayers->isItemChecked(ID_COMP_BASE+1));


  tron->saveColors(config);
  
  config->sync();
}
  

void App::menuCallback(int id)
{
  switch(id)
    {
    case ID_GAME_PAUSE:
      tron->togglePause();
      break;
    case ID_GAME_NEW:
      tron->newGame();
      break;
    case ID_GAME_QUIT:
      kapp->quit();
      break;
      
    case ID_COMP_BASE:
    case ID_COMP_BASE+1:
      {
	bool status=compPlayers->isItemChecked(id);
	tron->setComputerplayer(id-ID_COMP_BASE,status ? FALSE : TRUE);
	compPlayers->setItemChecked(id,status ? FALSE : TRUE);
	break;
      }
    case ID_STYLE_BASE:
    case ID_STYLE_BASE+1:
    case ID_STYLE_BASE+2:
    case ID_STYLE_BASE+3:
      {
	int i;
	if(id!=ID_STYLE_BASE+3) // pixstyle not implemented yet
	  {
	    for(i=ID_STYLE_BASE;i<ID_STYLE_BASE+4;i++)
	      styleMenu->setItemChecked(i,FALSE);
	    
	    styleMenu->setItemChecked(id,TRUE);
	  }

	if(id==ID_STYLE_BASE)
	  tron->setStyle(ORect);
	else if(id==ID_STYLE_BASE+1)
	  tron->setStyle(Rect);
	else if(id==ID_STYLE_BASE+2)
	  tron->setStyle(Circle);
	else if(id==ID_STYLE_BASE+3)
	  //tron->setStyle(Pix);
	  {
	    QString s("Not implemented yet :-(\n\nIf you have a suggestion about a well looking\npixmap for the players and the frame\n(Size from 4x4 to 16x16),\nit would be nice sending me a mail.\n(maybe with the pixmaps attached? :-) )\n") ;
	    QString t("\n<Matthias.Kiefer@stud.uni-karlsruhe.de>");
	    KMsgBox::message( this, "Sorry", (const char *)(s+t),
			      KMsgBox::INFORMATION, klocale->translate("OK") );
	  }
	break;
      }
    
    case ID_SIZE_BASE+4:
    case ID_SIZE_BASE+7:
    case ID_SIZE_BASE+10:
    case ID_SIZE_BASE+13:
    case ID_SIZE_BASE+16:
      {
	for(int i=ID_SIZE_BASE+4;i<ID_SIZE_BASE+17;i+=3)
	  sizeMenu->setItemChecked(i,FALSE);
	
	sizeMenu->setItemChecked(id,TRUE);

	tron->setRectSize(id-ID_SIZE_BASE);
	break;
      }


    case ID_OPT_KEYS:
      tron->configureKeys();
      break;
      
    case ID_OPT_SAVE:
      saveSettings();
      break;

    case ID_VIEW_TOOLBAR:
      {
	enableToolBar(); // toggles toolbar
	bool status=viewMenu->isItemChecked(ID_VIEW_TOOLBAR);
	viewMenu->setItemChecked(ID_VIEW_TOOLBAR, status?FALSE:TRUE);
	break;
      }

    case ID_VIEW_STATUSBAR:
      {
	enableStatusBar(); // toggles statusbar
	bool status=viewMenu->isItemChecked(ID_VIEW_STATUSBAR);
	viewMenu->setItemChecked(ID_VIEW_STATUSBAR, status?FALSE:TRUE);
	break;
      }

    case ID_COLOR_BASE:
    case ID_COLOR_BASE+1:
    case ID_COLOR_BASE+2:
    case ID_COLOR_BASE+3:
      tron->changeColor(id-ID_COLOR_BASE);
      break;

    case ID_VELOCITY_BASE+1:
    case ID_VELOCITY_BASE+2:
    case ID_VELOCITY_BASE+3:
    case ID_VELOCITY_BASE+4:
    case ID_VELOCITY_BASE+5:
    case ID_VELOCITY_BASE+6:
    case ID_VELOCITY_BASE+7:
    case ID_VELOCITY_BASE+8:
    case ID_VELOCITY_BASE+9:
      tron->setVelocity(id-ID_VELOCITY_BASE);
      updateVelocityMenu(id);
      break;

    case ID_HELP_ABOUT:
      {
	QString s;
	s.sprintf(klocale->translate("Tron v%s \n\n(C) 1998 by Matthias Kiefer \nemail:Matthias.Kiefer@stud.uni-karlsruhe.de\n"),VERSION);
	QString t(klocale->translate("\nAlgorithms for the computerplayer are from\nxtron-1.1 by Rhett D. Jacobs\n<rhett@hotel.canberra.edu.au>"));
	KMsgBox::message( this, klocale->translate("About Tron"), 
		       (const char *)(s+t),
		       KMsgBox::INFORMATION, klocale->translate("OK") );
	break;
      }
   
    case ID_HELP_CONTENTS:
      kapp->invokeHTMLHelp("","");
      break;

    }
}
  

void App::changeStatus(int player)
{
  static int player1Points,player2Points;
  QString s,t;
        
  // Wenn player=0, dann neues Spiel
  if(player==0)
    {
      player1Points=player2Points=0;
      s.sprintf(klocale->translate("Player 1: %d"),player1Points);
      statusbar->changeItem(s,ID_STATUS_BASE+1);
      s.sprintf(klocale->translate("Player 2: %d"),player2Points);
      statusbar->changeItem(s,ID_STATUS_BASE+2);
      statusbar->changeItem(" ",ID_STATUS_BASE+3);
    }
  else 
    {
      if(player==1)
	{
	  player1Points++;
	  s.sprintf(klocale->translate("Player 1: %d"),player1Points);
	  t=klocale->translate(" Player 2 crashed");
	  statusbar->changeItem(s,ID_STATUS_BASE+player);
	}
      else if(player==2)
	 {
	   player2Points++;
	   s.sprintf(klocale->translate("Player 2: %d"),player2Points);
	   t=klocale->translate(" Player 1 crashed");
	   statusbar->changeItem(s,ID_STATUS_BASE+player);
	 }

      else if(player==3)
	{
	   player1Points++;
	   player2Points++;
	   s.sprintf(klocale->translate("Player 1: %d"),player1Points);
	   statusbar->changeItem(s,ID_STATUS_BASE+1);
	   s.sprintf(klocale->translate("Player 2: %d"),player2Points);
	   statusbar->changeItem(s,ID_STATUS_BASE+2);

	   t=klocale->translate(" Both Players crashed");
	}

     
      statusbar->changeItem(t,ID_STATUS_BASE+3);
    }
}

void App::removeWinnerMessage()
{
  statusbar->changeItem(" ",ID_STATUS_BASE+3);
}



void App::barPositionChanged()
{
  KConfig *config=kapp->getConfig();
  KConfigGroupSaver saver(config,QString());
  config->writeEntry("MenuBarPos",(int)(menu->menuBarPos()));
  config->writeEntry("ToolbarPos",(int)(toolbar->barPos()));
}



void App::readProperties(KConfig *config)
{
  int i;
  bool visible=(bool)(config->readNumEntry("Toolbar",(int)(TRUE)));
  viewMenu->setItemChecked(ID_VIEW_TOOLBAR,visible);
    	  
  visible=(bool)(config->readNumEntry("Statusbar",(int)(TRUE)));
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR,visible);
  
  bool status=(bool)(config->readNumEntry("Computerplayer1",(int)(FALSE)));
  compPlayers->setItemChecked(ID_COMP_BASE,status);
  tron->setComputerplayer(0,status);
  status=(bool)(config->readNumEntry("Computerplayer2",(int)(FALSE)));
  compPlayers->setItemChecked(ID_COMP_BASE+1,status);
  tron->setComputerplayer(1,status);
  
  int velocity=config->readNumEntry("Velocity",5);
  tron->setVelocity(velocity);
  updateVelocityMenu(ID_VELOCITY_BASE+velocity); 

  TronStyle newStyle=(TronStyle)config->readNumEntry("Style",(int) Rect);
  tron->setStyle(newStyle);
  for(i=ID_STYLE_BASE;i<ID_STYLE_BASE+3;i++)
    styleMenu->setItemChecked(i,FALSE);

  switch(newStyle)
    {
    case ORect:
       styleMenu->setItemChecked(ID_STYLE_BASE,TRUE);
       break;
    case Rect:
       styleMenu->setItemChecked(ID_STYLE_BASE+1,TRUE);
       break;
    case Circle:
       styleMenu->setItemChecked(ID_STYLE_BASE+2,TRUE);
       break;
    case Pix:
       styleMenu->setItemChecked(ID_STYLE_BASE+3,TRUE);
       break;
    }

  int size=config->readNumEntry("RectSize",10);
  for(i=ID_SIZE_BASE+4;i<ID_SIZE_BASE+17;i+=3)
    sizeMenu->setItemChecked(i,FALSE);
  tron->setRectSize(size);
  sizeMenu->setItemChecked(size+ID_SIZE_BASE,TRUE);

  tron->restoreColors(config,TRUE);
  updateRects();
}


void App::saveProperties(KConfig *config)
{
  bool status=viewMenu->isItemChecked(ID_VIEW_TOOLBAR);
  config->writeEntry("Toolbar",(int)( status ));
  status=viewMenu->isItemChecked(ID_VIEW_STATUSBAR);
  config->writeEntry("Statusbar",(int)( status ));

  config->writeEntry("Velocity",tron->getVelocity());
  config->writeEntry("Style",(int)tron->getStyle());
  config->writeEntry("RectSize",tron->getRectSize());
  config->writeEntry("Computerplayer1"
		     ,(int)compPlayers->isItemChecked(ID_COMP_BASE));
  config->writeEntry("Computerplayer2"
		     ,(int)compPlayers->isItemChecked(ID_COMP_BASE+1));


  tron->saveColors(config,TRUE);
}


void App::placeTronWidget(bool fromResizeEvent)
{
 
  int rectSize=tron->getRectSize();

  int yOffset=(tron->height()%rectSize)/2;
  int xOffset=(tron->width()%rectSize)/2;

  int xBorder=0;
  int yBorder=0;

  if(toolbar->isVisible())
    {
      if(toolbar->barPos()==KToolBar::Bottom)
	yBorder+=toolbar->height();
      else if(toolbar->barPos()==KToolBar::Top)
	yOffset+=toolbar->height();
      else if(toolbar->barPos()==KToolBar::Left)
	xOffset+=toolbar->width();
      else if (toolbar->barPos()==KToolBar::Right)
	xBorder+=toolbar->width();

    }

  if(menu->menuBarPos()==KMenuBar::Top)
    yOffset+=menu->height();
  else  if(menu->menuBarPos()==KMenuBar::Bottom)
    yBorder+=menu->height();

  if(statusbar->isVisible())
    yBorder+=statusbar->height();
    
 
  
  if(fromResizeEvent)
    {
      xOffset+=(width()-xBorder-xOffset-tron->width())/2;
      yOffset+=(height()-yBorder-yOffset-tron->height())/2;
      tron->move(xOffset,yOffset);
    }
  else
    {
      /*
      tron->setGeometry(xOffset,yOffset,(width()-xBorder-xOffset)/rectSize*rectSize
      		,(height()-yBorder-yOffset)/rectSize*rectSize);
		*/

      updateRects();
    }
}


#include "app.moc"






