/********************************************************************************** 
  This file is part of the kde-game 'ktron'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <qevent.h>
#include <qkeycode.h>
#include <qimage.h>
#include <kapp.h>
#include <klocale.h>
#include <kkeyconf.h>
#include <kcolordlg.h>
#include <kmsgbox.h>
#include <sys/time.h>
#include <stdlib.h>

#include "tron.h"


Tron::Tron(QWidget *parent,const char *name) 
  : QWidget(parent,name)
{
  srand(time(0)); // used for computerplayers

  timer=new QTimer(this,"timer");
  connect(timer,SIGNAL(timeout()),SLOT(doMove()));
    
  KConfig *config=kapp->getConfig();
  if(config!=0)
    {
      restoreColors(config);
    }
  else
    {
      colors[0]=QColor("black");
      setPalette(colors[0]);
      colors[1]=QColor("red");
      colors[2]=QColor("blue");
      colors[3]=QColor("darkblue");
     }
 
  setFocusPolicy(QWidget::StrongFocus);
  
  KKeyConfig *keys = KKeyConfig::getKKeyConfig();
  if(keys)
    {
      keys->registerWidget("tron",this);
      keys->addKey("player1 up",Key_R);
      keys->addKey("player1 down",Key_C);
      keys->addKey("player1 right",Key_F);
      keys->addKey("player1 left",Key_D);
      keys->addKey("player1 Accel", Key_A);
      keys->addKey("player2 up",Key_Up);
      keys->addKey("player2 down",Key_Down);
      keys->addKey("player2 right",Key_Right);
      keys->addKey("player2 left",Key_Left);
      keys->addKey("player2 Accel", Key_Minus);
    }
    
  updateKeys();

  gameBlocked=FALSE;
  style=Rect;
  rectSize=8;
}


Tron::~Tron()
{
  if(playfield)
    {
      delete []  playfield;
      debug("playfield deleted");
    }
  if(timer)
    {
      delete timer;
      debug("timer deleted");
    }
  if(pixmap)
    delete pixmap;
}

void Tron::newGame()
{
  players[0].score=0;
  players[1].score=0;
  emit gameEnds(0);
  reset();
}

void Tron::reset()
{
  gamePaused=FALSE;
  gameEnded=TRUE;
  timer->stop(); 

  players[0].reset();
  players[1].reset();

  
  // Wenn playfield schon erstellt wurde, dann subern 
  // und Anfangskoordinaten festlegen;
  if(playfield)
    {
      int i;
      for(i=0;i<fieldWidth;i++)
	playfield[i].fill(0);
      
      // draw frame
      for(i=0;i<fieldWidth;i++)
	{
	  playfield[i][0]=3;
	  playfield[i][fieldHeight-1]=3;
	}
      for(i=0;i<fieldHeight;i++)
	{
	  playfield[0][i]=3;
	  playfield[fieldWidth-1][i]=3;
	}

      // Anfangskoordinaten setzen
 
      players[0].setCoordinates(fieldWidth/3, fieldHeight/2);
      players[1].setCoordinates(2*fieldWidth/3, fieldHeight/2);
      
      for(i=0;i<2;i++)
	playfield[players[i].xCoordinate][players[i].yCoordinate]=i+1;
      
      updatePixmap();
      update();
    }

  setFocus();

  emit gameReset();
}



void Tron::createNewPlayfield()
{
  if(playfield)
    {
      delete [] playfield;
    }

  if(pixmap)
    delete pixmap;
  
  // Gre des Feldes

  fieldWidth=width()/rectSize;
  fieldHeight=height()/rectSize;

  // Startpositionen;
  
  playfield=new QByteArray[fieldWidth];
  for(int i=0;i<fieldWidth;i++)
    playfield[i].resize(fieldHeight);

  pixmap=new QPixmap(fieldWidth*rectSize,fieldHeight*rectSize);
  pixmap->fill(this,0,0);

}

void Tron::updatePixmap()
{
  int i,j;

  pixmap->fill(this,0,0);

  QPainter p;  
  p.begin(pixmap);
  
  // alle Pixel prfen und evt. zeichen
  for(i=0;i<fieldWidth;i++)
	for(j=0;j<fieldHeight;j++)
	  {
	    if(playfield[i][j]!=0)
	      {
		drawRect(p,i,j,playfield[i][j]);
	      }
	  }
  p.end();

}


//neue Pixel der Spieler zeichnen

void Tron::paintPlayers()
{
  int i;
  QPainter p;
  p.begin(this);
  for(i=0;i<2;i++)
    {
      drawRect(p,players[i].xCoordinate,players[i].yCoordinate,i+1);
    }
  p.end();
  

  p.begin(pixmap);
  for(i=0;i<2;i++)
    {
      drawRect(p,players[i].xCoordinate,players[i].yCoordinate,i+1);
    }
  p.end();
}



void Tron::showWinner(int winner)
{
  int i,j;
  QPainter p;  
  
  emit gameEnds(winner);

  p.begin(pixmap);
  
  for(i=1;i<fieldWidth-1;i++)
    for(j=1;j<fieldHeight-1;j++)
      {
	if(playfield[i][j]!=0)
	  {
	    drawRect(p,i,j,winner);
	  }
      }

  p.end();

  repaint();
}





void Tron::drawRect(QPainter & p, int x, int y,int toDraw)
{
  switch(style)
    {
    case Pix:
      //p.drawPixmap(rectSize*x,rectSize*y,playerPix[toDraw]);
      break;

    case Rect:
      p.setBrush(colors[toDraw]);
      p.setPen(colors[toDraw]);
      p.drawRect(rectSize*x,rectSize*y,rectSize,rectSize);
      break;
      
    case Circle:
      p.setBrush(colors[toDraw]);
      p.setPen(colors[toDraw]);
      if(toDraw==3)
	p.drawRect(rectSize*x,rectSize*y,rectSize,rectSize);
      else
	p.drawEllipse(rectSize*x ,rectSize*y ,rectSize,rectSize);
      break;

    case ORect:
      p.setBrush(colors[toDraw]);
      p.setPen(colors[toDraw].light(180));
      p.drawRect(rectSize*x,rectSize*y,rectSize,rectSize);
      p.setPen(colors[0]);
      p.drawLine(rectSize*x,rectSize*y+rectSize-1,rectSize*x+rectSize-1
		 ,rectSize*y+rectSize-1);
      p.drawLine(rectSize*x,rectSize*y,rectSize*x,rectSize*y+rectSize-1);
      break;
    }
}
  
  



// configure Keys

void Tron::configureKeys()
{
  KKeyConfig *keys = KKeyConfig::getKKeyConfig();
  if(keys->configureKeys(this))
    updateKeys();
  
}


// change Keys
/*****************************************************************************
*** I know this is not a good style, but I'm just not very familiar with the *
*** keyconfig.                                                               *
*****************************************************************************/ 
void Tron::updateKeys()
{
  KKeyConfig *keys = KKeyConfig::getKKeyConfig();
  player1Up=keys->readCurrentKey("player1 up");
  player1Down=keys->readCurrentKey("player1 down");
  player1Right=keys->readCurrentKey("player1 right");
  player1Left=keys->readCurrentKey("player1 left");
  pl1AccKey=keys->readCurrentKey("player1 Accel");
  player2Up=keys->readCurrentKey("player2 up");
  player2Down=keys->readCurrentKey("player2 down");
  player2Right=keys->readCurrentKey("player2 right");
  player2Left=keys->readCurrentKey("player2 left");
  pl2AccKey=keys->readCurrentKey("player2 Accel");

}


// configure colors

void Tron::changeColor(int id)
{
  QColor color;
  int result=KColorDialog::getColor(color);
 
  if(result)  // if button 'ok' pressed
    {
      // backgroundcolor changed
      if(id==0)
	{
	  setPalette(color);
	  colors[0]=color;
	}
      else  // set other color
	{  
	  colors[id]=color;
	}
      updatePixmap();
      repaint();
     
    }
}

void Tron::saveColors(KConfig* config, bool sessionManagment)
{
  if(sessionManagment)
    {
      config->writeEntry("Color_Background",colors[0]);
      config->writeEntry("Color_Player1",colors[1]);
      config->writeEntry("Color_Player2",colors[2]);
      config->writeEntry("Color_Frame",colors[3]);
    }
  else
    {
      KConfigGroupSaver saver(config,"COLORS");

      config->writeEntry("Background",colors[0]);
      config->writeEntry("Player1",colors[1]);
      config->writeEntry("Player2",colors[2]);
      config->writeEntry("Frame",colors[3]);
      }
}

void Tron::restoreColors(KConfig *config, bool sessionManagment)
{
   if(sessionManagment)
    {
      colors[0]=config->readColorEntry("Color_Background",&QColor("black"));
      colors[1]=config->readColorEntry("Color_Player1",&QColor("red"));
      colors[2]=config->readColorEntry("Color_Player2",&QColor("blue"));
      colors[3]=config->readColorEntry("Color_Frame",&QColor("darkblue"));
    }
  else
    {
      KConfigGroupSaver saver(config,"COLORS");
      colors[0]=config->readColorEntry("Background",&QColor("black"));
      colors[1]=config->readColorEntry("Player1",&QColor("red"));
      colors[2]=config->readColorEntry("Player2",&QColor("blue"));
      colors[3]=config->readColorEntry("Frame",&QColor("darkblue"));
    }

   setPalette(colors[0]);
}



void Tron::togglePause()                 // pause or continue game
{
  if(!gameEnded)
    {
      if(gamePaused)
	{
	  gamePaused=FALSE;
	  update();
	  timer->start(velocity);
	}
      else 
	{
	  gamePaused=TRUE;
	  timer->stop();
	  update();
	}
    }
}



void Tron::setVelocity(int newVel)            // set new velocity
{
  //velocity=(10-newVel)*(10-newVel);
  velocity=(10-newVel)*20;

  if(!gameEnded && !gamePaused)
    timer->changeInterval(velocity);
 
}

int Tron::getVelocity()
{
  //return (int)(10-sqrt(velocity)); 
  return 10-(velocity/20);
}


void Tron::setStyle(TronStyle newStyle)
{
  style=newStyle;
  
  /*
    if(style==Pix)
    {
      playerPix[0]=QPixmap("/home/mk/player1.xpm");
      playerPix[1]=QPixmap("/home/mk/player2.xpm");
      playerPix[2]=QPixmap("/home/mk/frame.xpm");
    }
    */
 
  if(pixmap)
    {
      updatePixmap();
      repaint();
    }

}


TronStyle Tron::getStyle()
{
  return style;
}


void Tron::setRectSize(int newSize)
{
  if(newSize!=rectSize)
    {
      rectSize=newSize;
      createNewPlayfield();
      reset();
      emit sizeChanged();
    }  
}


int Tron::getRectSize()
{
  return rectSize;
}



bool Tron::crashed(int playerNr,int xInc, int yInc)
{
  if(playfield[players[playerNr].xCoordinate+xInc][players[playerNr].yCoordinate+yInc] !=0)
    return TRUE;

  else return FALSE;
}


void Tron::setComputerplayer(int playerNr, bool flag)
{
  if(playerNr<0 || playerNr > 1)
    return;
  
  players[playerNr].setComputer(flag);

}






// EVENTS


void Tron::paintEvent(QPaintEvent *)
{
  QPainter p;  
  
  p.begin(this);
  //  p.drawPixmap(0,0,*pixmap);
  bitBlt(this,0,0,pixmap);
  p.end();
 
  // Wenn Pause, Text auf Spielfeld schreiben

  if(gamePaused)
    drawText(width()/2-100,height()/2-4,
	     klocale->translate("game paused (Press CTRL+P)"));
  
}

void Tron::resizeEvent(QResizeEvent *)
{
  if(width()%rectSize !=0 || height()%rectSize != 0)
    {
      resize(width()/rectSize*rectSize,height()/rectSize*rectSize);
      emit sizeChanged(TRUE);  
    }
  else
    {
      createNewPlayfield();
      reset();
    }
}


void Tron::keyPressEvent(QKeyEvent *e)
{
  // Richtungen der Spieler anpassen
  int key=e->key();
  
  if(!players[1].computer)
    {
      if(key==player2Up)
	{
	  players[1].dir=Up;
	  players[1].keyPressed=TRUE;
	}
      else if(key==player2Left)
	{
	  players[1].dir=Left;
	  players[1].keyPressed=TRUE;
	}
      else if(key==player2Right)
	{
	  players[1].dir=Right;
	  players[1].keyPressed=TRUE;
	}
      else if(key==player2Down)
	{
	  players[1].dir=Down;
	  players[1].keyPressed=TRUE;
	}
      else if(key==pl2AccKey)
	{
	  players[1].accelerated=TRUE;
	}

    }
  
  if(!players[0].computer)
    {
      if(key==player1Left)
	{
	  players[0].dir=Left;
	  players[0].keyPressed=TRUE;
	}
      else if(key==player1Right)
	{
	  players[0].dir=Right;
	  players[0].keyPressed=TRUE;
	}
      else if(key==player1Up)
	{
	  players[0].dir=Up;
	  players[0].keyPressed=TRUE;
	}
      else if(key==player1Down)
	{
	  players[0].dir=Down;
	  players[0].keyPressed=TRUE;
	}

      else if(key==pl1AccKey)
	{
	  players[0].accelerated=TRUE;
	}
    }


  e->ignore();  // if key is unknown: ignore


  // if both players press keys at the same time, start game...

  if(gameEnded && !gameBlocked)
    {
      if(players[0].keyPressed && players[1].keyPressed)
	{
	   reset();
	   gameEnded=FALSE;
	   timer->start(velocity);
	}
    }

  // ...or continue
  else if(gamePaused)
    {
      if(players[0].keyPressed && players[1].keyPressed)
	{
	  gamePaused=FALSE;
	  timer->start(velocity);
	  update();
	}
    }
  

}

void Tron::keyReleaseEvent(QKeyEvent * e)
{
  int key=e->key();
  
  if(!players[1].computer)
    {
      if(key==pl2AccKey)
	{
	  players[1].accelerated=FALSE;
	  return;
	}
     
      else if(key==player2Left)
	{
	  players[1].keyPressed=FALSE;
	  return;
	}
      else if(key==player2Right)
	{
	  players[1].keyPressed=FALSE;
	  return;
	}
      else if(key==player2Up)
	{
	  players[1].keyPressed=FALSE;
	  return;
	}
      else if(key==player2Down)
	{
	  players[1].keyPressed=FALSE;
	  return;
	}
    }

 if(!players[0].computer)
   {
     if(key==player1Left)
       {
	 players[0].keyPressed=FALSE;
	 return;
       }
      else if(key==player1Right)
	{
	  players[0].keyPressed=FALSE;
	  return;
	}
      else if(key==player1Up)
	{
	  players[0].keyPressed=FALSE;
	  return;
	}
     else if(key==player1Down)
       {
	 players[0].keyPressed=FALSE;
	 return;
       }
      else if(key==pl1AccKey)
	{
	  players[0].accelerated=FALSE;
	  return;
	}
   }
  
 e->ignore();  // if pressed key is unknown, ignore it
        
}

// Wenn spielfeld aus dem Focus spiel anhalten

void Tron::focusOutEvent(QFocusEvent *)
{
  if(!gameEnded)
    {
      gamePaused=TRUE;
      timer->stop();
    }
}


//SLOTS


// doMove() wird von Qtimer timer aufgerufen

void Tron::doMove()
{
  int i;
  for(i=0;i<2;i++)
    {      
      // berprfen, ob Acceleratortaste gedrckt wurde...
      if(players[i].accelerated)
	{
	  switch(players[i].dir)
	    {
	    case Up:
	      if(crashed(i,0,-1))
		 players[i].alive=FALSE;
	      else players[i].yCoordinate--;
	      break;
	    case Down:
	    if(crashed(i,0,1))
	       players[i].alive=FALSE;
	    else players[i].yCoordinate++;
	      break;
	    case Left:
	       if(crashed(i,-1,0))
		 players[i].alive=FALSE;
	       else players[i].xCoordinate--; 
	      break;
	    case Right:
	      if(crashed(i,1,0))
		players[i].alive=FALSE;
	      else  players[i].xCoordinate++;
	      break;
	    }
	  
	  
	}
      playfield[players[i].xCoordinate][players[i].yCoordinate]=i+1;
      
    }

  if(players[0].accelerated || players[1].accelerated)
    {
       /* player collision check */
      if(!players[1].alive)
	{ 
	  int xInc=0,yInc=0;
	  switch(players[1].dir) 
	    {
	    case Left:
	      xInc = -1; break;
	    case Right:
	      xInc = 1; break;
	    case Up:
	      yInc = -1; break;
	    case Down:
	      yInc = 1; break;
	    }
	  if ((players[1].xCoordinate+xInc) == players[0].xCoordinate)   
	    if ((players[1].yCoordinate+yInc) == players[0].yCoordinate) 
	      {
		players[0].alive=FALSE;
	      }
	}
      
      paintPlayers();

      // crashtest
      if(!players[0].alive && !players[1].alive)
	{
	  timer->stop();
	  gameEnded=TRUE;
	  players[0].score++;
	  players[1].score++;
	  showWinner(3);
	}

      else
	for(i=0;i<2;i++)
	  {
	    if(!players[i].alive)
	      { 
		timer->stop();
		showWinner((i==0)? 2:1);
		gameEnded=TRUE;
		players[i].score++;
	      }
	  }
      

      if(gameEnded)
	{
	  //this is for waiting 0,5s before starting next game
	  gameBlocked=TRUE;
	  QTimer::singleShot(1000,this,SLOT(unblockGame()));
	  if(players[0].score >= 9 && players[1].score < 9)
	    {
	      QString s(klocale->translate("Winner is Player 2"));
	      KMsgBox::message(this, klocale->translate("Winner"), 
				(const char *)s,
				KMsgBox::INFORMATION, klocale->translate("OK") );
	      
	      newGame();
	    }
	  else if(players[1].score >= 9 && players[0].score < 9)
	    {
	      QString s(klocale->translate("Winner is Player 1"));
	      KMsgBox::message(this, klocale->translate("Winner"), 
			       (const char *)s,
			       KMsgBox::INFORMATION, klocale->translate("OK") );
	      
	      newGame();
	    }
	   else if(players[0].score >= 9 && players[1].score >= 9)
	     {
	      QString s(klocale->translate("Nobody won"));
	      KMsgBox::message(this, klocale->translate("Winner"), 
			       (const char *)s,
			       KMsgBox::INFORMATION, klocale->translate("OK") );
	      
	      newGame();
	    }
	}
    }
  
  // neue Spielerstandorte festlegen
  for(i=0;i<2;i++)
    {
      
      if(players[i].computer)
	think(i);

      switch(players[i].dir)
	{
	case Up:
	      if(crashed(i,0,-1))
		players[i].alive=FALSE;
	      else players[i].yCoordinate--;
	  break;
	case Down:
	      if(crashed(i,0,1))
		players[i].alive=FALSE;
	      else players[i].yCoordinate++;
	  break;
	case Left:
	      if(crashed(i,-1,0))
		players[i].alive=FALSE;
	      else players[i].xCoordinate--; 
	  break;
	case Right:
	      if(crashed(i,1,0))
		players[i].alive=FALSE;
	      else players[i].xCoordinate++;
	  break;
	}

      playfield[players[i].xCoordinate][players[i].yCoordinate]=i+1;

    }
  
  /* player collision check */
  if(!players[1].alive)
    { 
      int xInc=0,yInc=0;
      switch(players[1].dir) 
	{
	case Left:
	  xInc = -1; break;
	case Right:
	  xInc = 1; break;
	case Up:
	  yInc = -1; break;
	case Down:
	  yInc = 1; break;
	}
      if ((players[1].xCoordinate+xInc) == players[0].xCoordinate)   
	if ((players[1].yCoordinate+yInc) == players[0].yCoordinate) 
	  {
	    players[0].alive=FALSE;
	  }
    }
  
  paintPlayers();  
  
  if(!players[0].alive && !players[1].alive)
    {
      timer->stop();
      gameEnded=TRUE;
      players[0].score++;
      players[1].score++;
      showWinner(3);
    }
 
  else

    for(i=0;i<2;i++)
      {
	// crashtests
	if(!players[i].alive)
	  { 
	    timer->stop();
	    showWinner((i==0)? 2:1);
	    gameEnded=TRUE;
	    players[i].score++;
	  }
      }
  

  
  if(gameEnded)
    {
      //this is for waiting 0,5s before starting next game
      gameBlocked=TRUE;
      QTimer::singleShot(1000,this,SLOT(unblockGame()));

      if(players[0].score >= 9 && players[1].score < 9)
	{
	  QString s(klocale->translate("Winner is Player 2"));
	  KMsgBox::message(this, klocale->translate("Winner"), 
			   (const char *)s,
			   KMsgBox::INFORMATION, klocale->translate("OK") );
	  
	  newGame();
	}
      else if(players[1].score >= 9 && players[0].score < 9)
	{
	  QString s(klocale->translate("Winner is Player 1"));
	  KMsgBox::message(this, klocale->translate("Winner"), 
			   (const char *)s,
			   KMsgBox::INFORMATION, klocale->translate("OK") );
	  
	  newGame();
	}
      else if(players[0].score >= 9 && players[1].score >= 9)
	{
	  QString s(klocale->translate("Nobody won"));
	  KMsgBox::message(this, klocale->translate("Winner"), 
			   (const char *)s,
			   KMsgBox::INFORMATION, klocale->translate("OK") );
	  
	  newGame();
	}
    }    
    
}



void Tron::unblockGame()
{
  gameBlocked=FALSE;
}


//*****************************************************************************

 
/* artificial intelligence routines for computer player */
// This part is complete ported from  
// xtron-1.1 by Rhett D. Jacobs <rhett@hotel.canberra.edu.au>
void Tron::think(int playerNr)
{
  Direction sides[2];
  int flags[6] = {0,0,0,0,0,0};
  int index[2];
  int dis_forward,  dis_left, dis_right;
 
  dis_forward = dis_left = dis_right = 1;

  switch (players[playerNr].dir) {
  case Left:

    /* forward flags */
    flags[0] = -1;
    flags[1] = 0;

    /* left flags */
    flags[2] = 0;
    flags[3] = 1;

    /* right flags */
    flags[4] = 0;
    flags[5] = -1;

    /* turns to either side */
    sides[0] = Down;
    sides[1] = Up;
    break;
  case Right:
    flags[0] = 1;
    flags[1] = 0;
    flags[2] = 0;
    flags[3] = -1;
    flags[4] = 0;
    flags[5] = 1;
    sides[0] = Up;
    sides[1] = Down;
    break;
  case Up:
    flags[0] = 0;
    flags[1] = -1;
    flags[2] = -1;
    flags[3] = 0;
    flags[4] = 1;
    flags[5] = 0;
    sides[0] = Left;
    sides[1] = Right;
    break;
  case Down:
    flags[0] = 0;
    flags[1] = 1;
    flags[2] = 1;
    flags[3] = 0;
    flags[4] = -1;
    flags[5] = 0;
    sides[0] = Right;
    sides[1] = Left;
    break;
  }

  /* check forward */
  index[0] = players[playerNr].xCoordinate+flags[0];
  index[1] = players[playerNr].yCoordinate+flags[1];
  while (index[0] < fieldWidth-1 && index[0] >= 1 &&
	 index[1] < fieldHeight-1 && index[1] >= 1 &&
	 playfield[index[0]][index[1]] == 0) {
    dis_forward++;
    index[0] += flags[0];
    index[1] += flags[1];
  }
 
  if (dis_forward < 25) // change this to change level
    {   
      dis_forward = 100 - 100/dis_forward;

    /* check left */
    index[0] = players[playerNr].xCoordinate+flags[2];
    index[1] = players[playerNr].yCoordinate+flags[3];
    while (index[0] < fieldWidth-1 && index[0] >= 1 &&
	   index[1] < fieldHeight-1 && index[1] >= 1 &&
	   playfield[index[0]][index[1]] == 0) {
      dis_left++;
      index[0] += flags[2];
      index[1] += flags[3];
    } 
    
    /* check right */
    index[0] = players[playerNr].xCoordinate+flags[4];
    index[1] = players[playerNr].yCoordinate+flags[5];
    while (index[0] < fieldWidth-1 && index[0] >= 1 &&
	   index[1] <  fieldHeight-1 && index[1] >= 1 &&
	   playfield[index[0]][index[1]] == 0) {
      dis_right++;
      index[0] += flags[4];
      index[1] += flags[5];
    } 
    if(!(dis_left == 1 && dis_right == 1))
      if ((int)rand()%100 >= dis_forward || dis_forward == 0) {
       
	/* change direction */
	if ((int)rand()%100 <= (100*dis_left)/(dis_left+dis_right))
	  if (dis_left != 1)

	    /* turn to the left */
	    players[playerNr].dir = sides[0];
	  else

	    /* turn to the right */
	    players[playerNr].dir = sides[1];
	else
	  if (dis_right != 1)

	    /*  turn to the right */
	    players[playerNr].dir = sides[1];
	  else

	    /* turn to the left */
	    players[playerNr].dir = sides[0];
      }
  }
}




#include "tron.moc"






