/********************************************************************************** 
  This file is part of the kde-game 'ktron'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#ifndef TRON_H
#define TRON_H

#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qstring.h>
#include <math.h>

#include "player.h"

enum TronStyle{ORect , Rect , Circle , Pix};


class Tron : public QWidget
{
  Q_OBJECT

public:
  Tron(QWidget *parent=0, const char *name=0);
  ~Tron();
  void newGame();
  void togglePause();
  void setVelocity(int);
  int getVelocity();
  void configureKeys();
  void changeColor(int player);
  void saveColors(KConfig *config , bool sessionManagment=FALSE);
  void restoreColors(KConfig *config, bool sessionManagment=FALSE);
  void setComputerplayer(int playerNr, bool);
  void setStyle(TronStyle);
  TronStyle getStyle();
  void setRectSize(int newSize);
  int getRectSize();

signals:
  void gameEnds(int);  
  void sizeChanged(bool fromResizeEvent=FALSE);
  void gameReset();

protected:
  void paintEvent(QPaintEvent *);
  void resizeEvent(QResizeEvent *);
  void keyPressEvent(QKeyEvent *);
  void keyReleaseEvent(QKeyEvent *);
  void focusOutEvent(QFocusEvent *);

private:
  QPixmap *pixmap;
  QByteArray *playfield;  // Spielfield
  bool gamePaused,gameEnded;  // Flags
  bool gameBlocked; // used for waiting after game ended
  int fieldHeight,fieldWidth;   // Gre des Spielfelds
   QTimer *timer;
  int player1Up,player1Down,player1Right,player1Left,pl1AccKey;
  int player2Up,player2Down,player2Right,player2Left,pl2AccKey;
  player players[2];
  QColor colors[4]; //Farben der Spieler
  int velocity;             // Spielgeschwindigkeit	
  TronStyle style;
  int rectSize;
  QPixmap playerPix[3];

  // Funktionen
  void createNewPlayfield();
  void reset();
  void updatePixmap();
  void paintPlayers();
  void drawRect(QPainter & p, int x, int y,int toDraw);
  void showWinner(int winner);
  void updateKeys();
  bool crashed(int playerNr,int xInc, int yInc);
  void think(int playerNr);

  
private slots:
    void doMove();
    void unblockGame();
  };












#endif // TRON_H
